import numpy as np
import time
now=time.time()

#Main Magnet Current
MM=2973.77611556
#Horizontal Trim Quad Current
IQHC=-104.4215146
#Vertical Trim Quad Current
IQVC=877.9031788
#Arc Length of Dipole
LEND= 2.42
#Bend Angle from Dipole
ANGD= 0.174532925    
#Bending Radius from Dipole
RHO=LEND/ANGD
#Main Magnet Field Excitation
BDIPO=(10*(0.0009122+MM*2.371e-4+(MM**2)*1.717e-8-(MM**3)*2.412e-11 +(MM**4)*1.836e-14-(MM**5)*7.88e-18+(MM**6)*1.891e-21 -(MM**7)*2.351e-25+(MM**8)*1.163e-29))
#Magnetic Rigidity
BRHO=100*BDIPO*RHO
#Physical Length of Dipole
LD=2.41693*100

RD=10

#Length of first drift
ld1=99.61422
#Length of second drift
ld2=28.9875

#Length of short quadrupoles
QS=49.3
#Length of long quadrupoles
QL=50.4

#Radius to poletip in quadrupoles
BR=16.5/2

#Main horizontal quadrupole current
IQH=MM+0.20022*IQHC
#Main vertical quadrupole current
IQV=MM-0.20022*IQVC
#Quadrupoles have five windings: four from the main magnet, one from the trim supplies

#Horizontal quadrupole field excitation at poletip
bqh=10*(0.001818+(IQH*9.080e-4)+(IQH**2*6.657e-9)-(IQH**3*7.225e-12)+(IQH**4*3.239e-15)-(IQH**5*5.07e-19))
#Vertical quadrupole field excitation at poletip
bqv=10*(0.002099+(IQV*9.257e-4)+(IQV**2*1.164e-8)-(IQV**3*1.046e-11)+(IQV**4*4.057e-15)-(IQV**5*5.75e-19))

#Quadrupole strengths
K1QH=1.00*bqh/(BRHO*QS)
K1QV=1.0030*bqv/(BRHO*QL)
	
#Quadrupole 
BQH=(BR)*BRHO*K1QH
BQV=-(BR)*BRHO*K1QV


xpas=1

#Start your lattice by generating an accelerator handle that can have items added to it.
DIPO=BEND('BEND scal',XL=LD, B1=BDIPO, R_0=RD, XPAS=xpas, KPOS=3)
QF  =QUADRUPO('QF scal',XL=QS, R_0=BR, B_0=BQH, XPAS=xpas, KPOS=1)
QD  =QUADRUPO('QD scal',XL=QL, R_0=BR, B_0=BQV, XPAS=xpas, KPOS=1)
dr1 =DRIFT('Drift 1', XL=ld1)
dr2 =DRIFT('Drift 2', XL=ld2)
dr3 =DRIFT('Drift 3', XL=LD)

#Generate a dictionary of FODO cells, there are 8 disctinct kinds that comprise a superperiod
cell={}
for i in range(1,9):
	cell[i]=Line('cell%s'%i)
cell[1].add(dr1,QD,dr2,DIPO)
cell[2].add(dr1,QF,dr2,DIPO)
cell[3].add(dr1,QD,dr2,dr3)
cell[4].add(dr1,QF,dr2,DIPO)
cell[5].add(dr1,QD,dr2,DIPO)
cell[6].add(dr1,QF,dr2,dr3)
cell[7].add(dr1,QD,dr2,DIPO)
cell[8].add(dr1,QF,dr2,DIPO)

#Add the FODO cells to the superperiod
SP=Line('SP')
[SP.add(cell[i]) for i in cell]

#Add the 6 superperiods to the lattice
ring=Line('Ring')
[ring.add(SP) for i in range(0,6)]
	
scl=2.0
scal=SCALING(IOPT=1, NFAM=1,NAMEF="scal",NT=-1,SCL=scl,TIM=1)

#
bstr=Line('Booster')
ob=OBJET2()
ob.set(BORO=BRHO)
ob.add(Y=0.,D=1.)

bstr.add(ob)
bstr.add(PROTON())
bstr.add(FAISTORE(FNAME='zgoubi.fai',IP=1))
bstr.add(ring)
bstr.add(FAISCEAU())
bstr.add(END())
#print bstr

print time.time()-now
#Run your lattice
r=bstr.run(xterm=False)
#Save your results
print time.time()-now
r.save_res("bstr.res")
print time.time()-now

#%%
'''
#Calculating the betatron tunes
bstr=Line('Booster')
ob5=OBJET5()
ob5.set(BORO=BRHO)
ob5.set(YR=0.,TR=0., ZR=0., PR=0., DR=1.0)
ob5.set(PY=1.e-3, PT=1.e-2, PZ=1.e-3, PP=1.e-2, PX=0., PD=1.e-4)
bstr.add(ob5)
bstr.add(ring)
matrix=MATRIX(IORD=1, IFOC=11)
bstr.add(matrix)
bstr.add(END())
#bstr.full_tracking(True)
r=bstr.run()
r.save_res('bstr2.res')
tune=r.get_tune()
print tune
#print twissparam

#'''
'''
#get twiss parameters at end of cell, returns [beta_y,alpha_y,gamma_y,disp_y,disp_py,beta_z,alpha_z,gamma_z,disp_z,disp_pz]
#twissparam = r.get_twiss_parameters()
#betayz = [twissparam['beta_y'][0],twissparam['beta_z'][0]]
#alphayz = [twissparam['alpha_y'][0],twissparam['alpha_z'][0]]
#gammayz = [twissparam['gamma_y'][0],twissparam['gamma_z'][0]]
#get periodic twiss parameters and dispersion
#bstr.full_tracking(True)
#twiss_profiles = get_twiss_profiles(bstr,'twiss_profiles.txt')

#'''

#%%
'''
#Twiss parameter calculations
twss=TWISS(KTW1=2, FAC_D=1, FAC_A=1)
bstr=Line('Booster')
bstr.add(ob5)
bstr.add(ring)
bstr.add(twss)
bstr.add(END())
r=bstr.run()
r.save_res('bstr3.res')
r.save_twiss('zgoubi.twiss')
#'''
#%%
'''
#Twiss parameter calculations
twss=TWISS(KTW1=2, FAC_D=1, FAC_A=1)
bstr=Line('Booster')
ob5.set(YR=0.,TR=0., ZR=0., PR=0., DR=1.01)
bstr.add(ob5)
bstr.add(ring)
bstr.add(FCO())
bstr.add(twss)
bstr.add(FAISCEAU())
bstr.add(END())
r=bstr.run()
r.save_res('bstr4.res')
r.save_twiss('zgoubi.twiss')
twissparam = r.get_twiss_parameters()#'''
#%%
#
'''
bstr=Line('Booster')
bstr.add(ob)
bstr.add(PROTON())
bstr.add(FAISTORE(FNAME='zgoubi.fai',IP=1))
bstr.add(ring)
bstr.add(FAISCEAU())
bstr.add(FCO())
bstr.add(END())

print time.time()-now
#Run your lattice
r=bstr.run(xterm=False)
bstr=Line('Booster')
bstr.add(ob)
bstr.add(PROTON())
#bstr.add(FAISTORE(FNAME='zgoubi.fai',IP=1))
bstr.add(ring)
#bstr.add(FAISCEAU())
#bstr.add(FCO())
bstr.add(END())
find_closed_orbit(bstr, init_YTZP=[0,0,0,0], tol=1e-6)


#'''
#%%
'''
#Multi-turn spin tracking
cavV=32e3
sigS=np.pi/6

cav=CAVITE(IOPT=2,L=201.78,h=2.00,V=cavV,sig_s=sigS)
reb=REBELOTE(NPASS=99,KWRIT=0.3,K=99)

st=SPNTRK()
st.set(KSO=3)

bstr=Line('Booster')
bstr.add(ob)
bstr.add(HELION())
bstr.add(FAISTORE(FNAME='zgoubi.fai',IP=1))
bstr.add(st)
bstr.add(ring)
bstr.add(cav)
bstr.add(reb)
bstr.add(END())
r=bstr.run()
r.save_fai('bstr.fai')

#'''



